#!/usr/bin/env python

# kensentme.cgi by Charl P. Botha http://cpbotha.net/

# based on
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/273844

# written many years ago because academic colleagues kept asking me to
# download papers for them. I wrote this script to automate the
# actions I would manually do for them in any case.

# Quickstart

# 1. add approved email addresses to the set "allowed_addresses".
#    Downloaded full texts will only be sent to these addresses.
# 2. Put the script somewhere where your academic web server will
#    see it as a CGI, usually in ~/public_html/cgi/
# 3. `chmod +x kensentme.cgi`

import cgi
# enable cgi traceback manager.
import cgitb; cgitb.enable()
import os, sys
import logging
import re
import tempfile
import urlparse
import urllib2

# Python set containing all approved email addresses
# * comma-separated
# * NB: all in lowercase
allowed_addresses = {
'someperson@somedomain.com',
'someotherperson@someotherdomain.com'
}


HTML_TEMPLATE = \
"""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html><head><title>ken sent you, right?</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
</head><body><h1>ken sent you, right?</h1>

<p>Paste the URL of the PDF that you would like into the PDF URL text
box and paste your email address into the, err, email address box.
If you're in the list, your article will be mailed to you.
</p>

<hr>
<form action="%(SCRIPT_NAME)s" method="POST" enctype="multipart/form-data">
<table width="100%%">

<tr><td width="20%%">PDF URL:</td>
<td width="80%%"><input style="width: 100%%" name="article_url" type="text"></td></tr>

<tr><td>Your email address:</td>
<td><input style="width: 100%%" name="email_address" type="text"></td></tr>

</table>
<input name="submit" type="submit" value="Send me my article"><br>

<p>
Please only click the button once for each request.  After clicking,
it could take a while for the next page to appear.  This delay is
dependent on the size of the article.
</p>

<p>
Pro tips:
    <ul>
    <li>If you have the IEEE "arnumber" (easily found in the article overview URL), get the PDF by entering the shortcut http://ieee.org/12345 where of course you replace 12345 with the arnumber.</li>
    </ul>
</p>

</form>
<hr>

</body>
</html>"""

END_MSG = \
"""NEWS: 
 * It seems ieeexplore are hiding their articles better.  We are now
   better at decoding their URLs. :)
 * Certain sites were filtering based on the browser software being
   used.  We've fixed that too.
"""


def print_html_form ():
    """This prints out the html form. Note that the action is set to
      the name of the script which makes this is a self-posting form.
      In other words, this cgi both displays a form and processes it.
    """
    print "content-type: text/html\n"
    print HTML_TEMPLATE % {'SCRIPT_NAME':os.environ['SCRIPT_NAME']}

def send_article(url, email, logger):
    pu = urlparse.urlparse(url)
    if pu[0].startswith('http') and pu[1] and pu[2]:
        # if scheme, netloc and path are good, we do it...
        # we could test for netloc (hostname) in a list

        
        # if there are more transforms, move out into separate
        # function.

        # on ieeexplore
        # stamp/stamp.jsp?tp=&arnumber=4492774&isnumber=4675189 
        # should become
        # stampPDF/getPDF.jsp?tp=&arnumber=....&isnumber=...
        # the tp=& is new since 20090514
        mo = re.match(
                'http://ieeexplore.ieee.org/stamp/stamp.jsp\?(.*)',
                url)
        if mo:
            url = \
            'http://ieeexplore.ieee.org/stampPDF/getPDF.jsp?%s' % (mo.groups()[0],)
            logger.info('Transformed IEEE URL.')

        # also on ieeexplore
        # iel5/72/27005/01199651.pdf
        # should become
        # ielx5/72/27005/01199651.pdf?tp=&arnumber=1199651&isnumber=27005
        mo = re.match(
                'http://ieeexplore.ieee.org/iel5/([0-9]+)/([0-9]+)/([0-9]+).pdf',
                url)

        if mo:
            mysnum, isnum, arnum  = mo.groups()
            url = \
            'http://ieeexplore.ieee.org/ielx5/%s/%s/%s.pdf?tp=&arnumber=%s&isnumber=%s' % (mysnum, isnum, arnum, arnum, isnum)
            logger.info('Transformed IEEE URL version 2.')


        # shorthand:
        # ieeearnumber=5473227 will become:
        # http://ieeexplore.ieee.org/stampPDF/getPDF.jsp?tp=&arnumber=5473227
        mo = re.match(
                'http://ieee.org/(.*)', url)
        if mo:
            url = \
            'http://ieeexplore.ieee.org/stampPDF/getPDF.jsp?tp=&arnumber=%s' % (mo.groups()[0],)
            logger.info('Transformed IEEE shortcut to %s.' % (url,))

        # here used to be old_getter_code()

        # create temporary name and write the thingy
        tname = tempfile.mktemp(suffix='.pdf')

        # now get wget to do the hard work
        user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.4) Gecko/2008102920 Firefox/3.0.4'
        ret = os.system('wget "%s" -O "%s" -U "%s"' % (url, tname, user_agent))

        # then check that the retrieved file is a valid PDF
        p = os.popen('file -b %s' % (tname,))

        if not p.read().startswith('PDF'):
            raise RuntimeError('PDF could not be retrieved. Please try again later OR fix your PDF URL.')

        logger.info("wrote article to %s" % (tname,))
        
        # now use mutt to send it
        ret = os.system('echo "your article: %s" | mutt -s ' \
                  '"it seems that ken did send you (AUTO)" -a %s -- %s' % \
                  (url, tname, email))

        if ret != 0:
            raise RuntimeError('Error mailing attachment.')
        else:
            logger.info("mutt returned %d" % (ret,))

    else:
        raise RuntimeError('Invalid URL specified.')

def old_getter_code():
        request = urllib2.Request(url)
        request.add_header('User-Agent', 
        'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.4) Gecko/2008102920 Firefox/3.0.4'
        )

        try:
            # open the url
            ufo = urllib2.urlopen(request)
        except urllib2.HTTPError, e:
            emsg = 'HTTP error retrieving URL: %s.' % (str(e),)
            logger.info(emsg)
            raise RuntimeError(emsg)
            


        
        # make sure it's pdf (add more types later)
        ct = ufo.info().gettype()
        logger.info(ct)
        if ct not in ('application/pdf',) and ufo.geturl() != url:
            logger.info('URL %s has incorrect content type.' % (url,))

            # let's see if it's a known error
            ad = ufo.read()
            if ad.find(
                    'All online seats are currently occupied.') >= 0:
                msg = 'All IEEE licenses currently in use for %s.'
                raise RuntimeError(msg % (url,))
                        

            # unknown error, flag it.
            raise RuntimeError(
            'There is no PDF at %s, or service down.' % (url,))

        else:
            logger.info('Handling URL %s.' % (url,))
        
        # read from it
        ad = ufo.read()
        
        # create temporary name and write the thingy
        tname = tempfile.mktemp(suffix='.pdf')
        tfile = open(tname, 'w')
        tfile.write(ad)
        tfile.close()

        logger.info("wrote article to %s" % (tname,))
        
        # now use mutt to send it
        os.system('echo "your article: %s" | mutt -a %s -s ' \
                  '"it seems that ken did send you (AUTO)" %s' % \
                  (url, tname, email))


    
def process_request():
           
    form = cgi.FieldStorage()
    if not form.has_key('article_url'):
        return False

    article_url = form['article_url'].value
    email_address = form['email_address'].value

    # setup logging
    logger = logging.getLogger('kensentme')
    hdlr = logging.FileHandler(
            '/home/cpbotha/public_html/cgi-bin/kensentme.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    logger.addHandler(hdlr)
    logger.setLevel(logging.DEBUG)

    ff = email_address
    if ff.lower() not in allowed_addresses:
        logger.info("Refusing to help %s with article." % (ff,))
        print "content-type: text/plain\n"
        print "You're not in the list.  Sorry."


    else:
        logger.info("Helping %s with article." % (ff,))

        try:
            send_article(article_url, email_address, logger)

        except RuntimeError, e:
            print "content-type: text/plain\n"
            print "ERROR! Could not send article: %s" % (str(e),)
            print "Use your browser's back button to try again."

        else:
            print "content-type: text/plain\n"
            print "SUCCESS!  You should be receiving an email soon."
            print "Use your browser's back button to get another article."
            print ""
            print END_MSG


    
    return True

if not process_request():
    print_html_form ()
